@@ -2,7 +2,7 @@
  * vlc.c: the VLC player
  *****************************************************************************
  * Copyright (C) 1998-2008 the VideoLAN team
- * $Id: 200060fcf8d9a5c5511c653fd52147d31f64802d $
+ * $Id: 4d20f89069d686e20a4f5cb5451cf9b9d7073a42 $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -32,24 +32,29 @@
 #include <vlc/vlc.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdbool.h>
 #include <locale.h>
+#include <signal.h>
+#ifdef HAVE_PTHREAD_H
+# include <pthread.h>
+#endif
+#include <unistd.h>
 
 #ifdef __APPLE__
 #include <string.h>
 #endif
 
+#ifdef __OS2__
+# define pthread_t      int
+# define pthread_self() _gettid()
+#endif
+
 
 /* Explicit HACK */
 extern void LocaleFree (const char *);
 extern char *FromLocale (const char *);
 extern void vlc_enable_override (void);
 
-#include <signal.h>
-#include <time.h>
-#include <pthread.h>
-#include <unistd.h>
-#include <dlfcn.h>
-
 #ifdef HAVE_MAEMO
 static void dummy_handler (int signum)
 {
@@ -57,34 +62,63 @@ static void dummy_handler (int signum)
 }
 #endif
 
+static bool signal_ignored (int signum)
+{
+    struct sigaction sa;
+
+    if (sigaction (signum, NULL, &sa))
+        return false;
+    return ((sa.sa_flags & SA_SIGINFO)
+            ? (void *)sa.sa_sigaction : (void *)sa.sa_handler) == SIG_IGN;
+}
+
+static void vlc_kill (void *data)
+{
+#ifndef __OS2__
+    pthread_t *ps = data;
+
+    pthread_kill (*ps, SIGTERM);
+#else
+    // send a signal to the main thread
+    kill (getpid(), SIGTERM);
+#endif
+}
+
+static void exit_timeout (int signum)
+{
+    (void) signum;
+    signal (SIGINT, SIG_DFL);
+}
+
 /*****************************************************************************
  * main: parse command line, start interface and spawn threads.
  *****************************************************************************/
 int main( int i_argc, const char *ppsz_argv[] )
 {
     /* The so-called POSIX-compliant MacOS X reportedly processes SIGPIPE even
-     * if it is blocked in all thread. Also some libraries want SIGPIPE blocked
-     * as they have no clue about signal masks.
+     * if it is blocked in all thread.
      * Note: this is NOT an excuse for not protecting against SIGPIPE. If
      * LibVLC runs outside of VLC, we cannot rely on this code snippet. */
     signal (SIGPIPE, SIG_IGN);
-    /* Restore default for SIGCHLD in case parent ignores it. */
+    /* Restore SIGCHLD in case our parent process ignores it. */
     signal (SIGCHLD, SIG_DFL);
 
-#ifdef HAVE_SETENV
-# ifndef NDEBUG
+#ifndef NDEBUG
     /* Activate malloc checking routines to detect heap corruptions. */
     setenv ("MALLOC_CHECK_", "2", 1);
 
     /* Disable the ugly Gnome crash dialog so that we properly segfault */
     setenv ("GNOME_DISABLE_CRASH_DIALOG", "1", 1);
-# endif
+#endif
+
+#ifdef TOP_BUILDDIR
+    setenv ("VLC_PLUGIN_PATH", TOP_BUILDDIR"/modules", 1);
+#endif
 
     /* Clear the X.Org startup notification ID. Otherwise the UI might try to
      * change the environment while the process is multi-threaded. That could
      * crash. Screw you X.Org. Next time write a thread-safe specification. */
     unsetenv ("DESKTOP_STARTUP_ID");
-#endif
 
 #ifndef ALLOW_RUN_AS_ROOT
     if (geteuid () == 0)
@@ -100,42 +134,49 @@ int main( int i_argc, const char *ppsz_argv[] )
     setlocale (LC_ALL, "");
 
     if (isatty (STDERR_FILENO))
-        /* This message clutters error logs. It is print it only on a TTY.
-         * Forunately, LibVLC prints version infos with -vv anyhow. */
+        /* This message clutters error logs. It is printed only on a TTY.
+         * Fortunately, LibVLC prints version info with -vv anyway. */
         fprintf (stderr, "VLC media player %s (revision %s)\n",
                  libvlc_get_version(), libvlc_get_changeset());
 
-    /* Synchronously intercepted POSIX signals.
-     *
-     * In a threaded program such as VLC, the only sane way to handle signals
-     * is to block them in all threads but one - this is the only way to
-     * predict which thread will receive them. If any piece of code depends
-     * on delivery of one of this signal it is intrinsically not thread-safe
-     * and MUST NOT be used in VLC, whether we like it or not.
-     * There is only one exception: if the signal is raised with
-     * pthread_kill() - we do not use this in LibVLC but some pthread
-     * implementations use them internally. You should really use conditions
-     * for thread synchronization anyway.
-     *
-     * Signal that request a clean shutdown, and force an unclean shutdown
-     * if they are triggered again 2+ seconds later.
-     * We have to handle SIGTERM cleanly because of daemon mode.
-     * Note that we set the signals after the vlc_create call. */
-    static const int sigs[] = {
-        SIGINT, SIGHUP, SIGQUIT, SIGTERM,
-    /* Signals that cause a no-op:
-     * - SIGPIPE might happen with sockets and would crash VLC. It MUST be
-     *   blocked by any LibVLC-dependent application, not just VLC.
-     * - SIGCHLD comes after exec*() (such as httpd CGI support) and must
-     *   be dequeued to cleanup zombie processes.
-     */
-        SIGPIPE, SIGCHLD
-    };
-
     sigset_t set;
+
     sigemptyset (&set);
-    for (unsigned i = 0; i < sizeof (sigs) / sizeof (sigs[0]); i++)
-        sigaddset (&set, sigs[i]);
+    /* VLC uses sigwait() to dequeue interesting signals.
+     * For this to work, those signals must be blocked in all threads,
+     * including the thread calling sigwait() (see the man page for details).
+     *
+     * There are two advantages to sigwait() over traditional signal handlers:
+     *  - delivery is synchronous: no need to worry about async-safety,
+     *  - EINTR is not generated: other threads need not handle that error.
+     * That being said, some LibVLC programs do not use sigwait(). Therefore
+     * EINTR must still be handled cleanly, notably from poll() calls.
+     *
+     * Signals that request a clean shutdown, and force an unclean shutdown
+     * if they are triggered again 2+ seconds later.
+     * We have to handle SIGTERM cleanly because of daemon mode. */
+    sigaddset (&set, SIGINT);
+    sigaddset (&set, SIGHUP);
+    sigaddset (&set, SIGQUIT);
+    sigaddset (&set, SIGTERM);
+
+    /* SIGPIPE can happen and would crash the process. On modern systems,
+     * the MSG_NOSIGNAL flag protects socket write operations against SIGPIPE.
+     * But we still need to block SIGPIPE when:
+     *  - writing to pipes,
+     *  - using write() instead of send() for code not specific to sockets.
+     * LibVLC code assumes that SIGPIPE is blocked. Other LibVLC applications
+     * shall block it (or handle it somehow) too.
+     */
+    sigaddset (&set, SIGPIPE);
+
+    /* SIGCHLD must be dequeued to clean up zombie child processes.
+     * Furthermore the handler must not be set to SIG_IGN (see above).
+     * We cannot pragmatically handle EINTR, short reads and short writes
+     * in every code paths (including underlying libraries). So we just
+     * block SIGCHLD in all threads, and dequeue it below. */
+    sigaddset (&set, SIGCHLD);
+
 #ifdef HAVE_MAEMO
     sigaddset (&set, SIGRTMIN);
     {
@@ -143,20 +184,15 @@ int main( int i_argc, const char *ppsz_argv[] )
         sigaction (SIGRTMIN, &act, NULL);
     }
 #endif
-
     /* Block all these signals */
-    pthread_sigmask (SIG_BLOCK, &set, NULL);
-    sigdelset (&set, SIGPIPE);
-    sigdelset (&set, SIGCHLD);
+    pthread_sigmask (SIG_SETMASK, &set, NULL);
 
     /* Note that FromLocale() can be used before libvlc is initialized */
     const char *argv[i_argc + 3];
     int argc = 0;
 
     argv[argc++] = "--no-ignore-config";
-#ifdef TOP_BUILDDIR
-    argv[argc++] = FromLocale ("--plugin-path="TOP_BUILDDIR"/modules");
-#endif
+    argv[argc++] = "--media-library";
 #ifdef TOP_SRCDIR
     argv[argc++] = FromLocale ("--data-path="TOP_SRCDIR"/share");
 #endif
@@ -179,25 +215,51 @@ int main( int i_argc, const char *ppsz_argv[] )
 
     /* Initialize libvlc */
     libvlc_instance_t *vlc = libvlc_new (argc, argv);
+    if (vlc == NULL)
+        goto out;
 
-    if (vlc != NULL)
-    {
-        libvlc_set_user_agent (vlc, "VLC media player", NULL);
+    libvlc_set_user_agent (vlc, "VLC media player", "VLC/"PACKAGE_VERSION);
 
-        if (libvlc_add_intf (vlc, "signals"))
-            pthread_sigmask (SIG_UNBLOCK, &set, NULL);
-#if !defined (HAVE_MAEMO) && !defined __APPLE__
-        libvlc_add_intf (vlc, "globalhotkeys,none");
+#if !defined (HAVE_MAEMO) && !defined __APPLE__ && !defined (__OS2__)
+    libvlc_add_intf (vlc, "globalhotkeys,none");
 #endif
-        if (libvlc_add_intf (vlc, NULL) == 0)
-        {
-            libvlc_playlist_play (vlc, -1, 0, NULL);
-            libvlc_wait (vlc);
-        }
-        libvlc_release (vlc);
-    }
+    if (libvlc_add_intf (vlc, NULL))
+        goto out;
 
-    for (int i = 1; i < argc; i++)
+    libvlc_playlist_play (vlc, -1, 0, NULL);
+
+    /* Wait for a termination signal */
+    pthread_t self = pthread_self ();
+    libvlc_set_exit_handler (vlc, vlc_kill, &self);
+
+    /* Qt4 insists on catching SIGCHLD via signal handler. To work around that,
+     * unblock it after all our child threads are created. */
+    sigdelset (&set, SIGCHLD);
+    pthread_sigmask (SIG_SETMASK, &set, NULL);
+
+    /* Do not dequeue SIGHUP if it is ignored (nohup) */
+    if (signal_ignored (SIGHUP))
+        sigdelset (&set, SIGHUP);
+    /* Ignore SIGPIPE */
+    sigdelset (&set, SIGPIPE);
+
+    int signum;
+    sigwait (&set, &signum);
+
+    /* Restore default signal behaviour after 3 seconds */
+    sigemptyset (&set);
+    sigaddset (&set, SIGINT);
+    sigaddset (&set, SIGALRM);
+    signal (SIGINT, SIG_IGN);
+    signal (SIGALRM, exit_timeout);
+    pthread_sigmask (SIG_UNBLOCK, &set, NULL);
+    alarm (3);
+
+    /* Cleanup */
+out:
+    if (vlc != NULL)
+        libvlc_release (vlc);
+    for (int i = 2; i < argc; i++)
         LocaleFree (argv[i]);
 
     return 0;
