@@ -2,7 +2,7 @@
  * misc.m: code not specific to vlc
  *****************************************************************************
  * Copyright (C) 2003-2011 VLC authors and VideoLAN
- * $Id: 9c313b0c3f228ec587e9cb9bda7f0ebadf546677 $
+ * $Id: a134f5b27c9d51b8d992d239c29f81f8f34a7cbc $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Felix Paul Kühne <fkuehne at videolan dot org>
@@ -180,7 +180,6 @@ static NSMutableArray *blackoutWindows = NULL;
     self = [super initWithContentRect:contentRect styleMask:styleMask backing:backingType defer:flag];
     if( self )
     {
-        b_isFullscreen = NO;
         b_isset_canBecomeKeyWindow = NO;
         /* we don't want this window to be restored on relaunch */
         if (OSX_LION)
@@ -369,16 +368,6 @@ static NSMutableArray *blackoutWindows = NULL;
     }
 }
 
-- (void)setFullscreen:(BOOL)b_var
-{
-    b_isFullscreen = b_var;
-}
-
-- (BOOL)isFullscreen
-{
-    return b_isFullscreen;
-}
-
 - (IBAction)fullscreen:(id)sender
 {
     [[VLCCoreInteraction sharedInstance] toggleFullscreen];
