@@ -138,9 +138,21 @@ function dirlisting(url,listing,acl_)
 end
 
 -- FIXME: Experimental art support. Needs some cleaning up.
-function callback_art(data, request)
+function callback_art(data, request, args)
     local art = function(data, request)
-        local item = vlc.input.item()
+        local num = nil
+        if args ~= nil then
+            num = string.gmatch(args, "item=(.*)")
+            if num ~= nil then
+                num = num()
+            end
+        end
+        local item
+        if num == nil then
+            item = vlc.input.item()
+        else
+            item = vlc.playlist.get(num).item
+        end
         local metas = item:metas()
         local filename = vlc.strings.decode_uri(string.gsub(metas["artwork_url"],"file://",""))
         local size = vlc.net.stat(filename).size
